#!/bin/bash

##
## This script controls the lifecycle of the server and
## provides methods for starting, stopping, restarting
## and upgrading the server.
##
## This script is not meant to be run outside of the
## main installation directory.
##
## Author: Arthur Heffern
## Maintainer: Michael Yara
## Last Modified: Jan 6, 2014
##

# Trap SIGHUP
trap "" SIGHUP

# Setup variables for execution
MIN_JRE_VERSION="1.7"
INSTALLDIR=`pwd`
BASEDIR=`dirname ${0}`
BASENAME=`basename ${0} .sh`
CLASSPATH=`find modules ${INSTALLDIR}/modules -type d -name shared | grep '^/' | sed 's/$/\/\*/g' | tr '\n' ':'`
OKSTATUSINDICATOR="OK"
. ${BASEDIR}/config-vars.sh
PATH=$PATH:/sbin
ARG2=${2}

#
# Function used to run the install.sh from the upgrade directory
# in order to apply a downloaded patch to the server.
# This function returns exit code 128 in order to inform startup.sh
# that an upgrade was just applied and the server needs to be restarted.
# 
function performUpgrade {
	echo "--PERFORMING UPGRADE--" >> ${BASEDIR}/log/${BASENAME}.out.log
	if [ -e  update/system/install.sh ]; then
		cd update/system/
		/bin/sh ./install.sh
		exit 128
	fi
}

#
# Function used to stop the server.
#
function stop {

    echo -n "Stopping ${BASENAME}... "

	PID=$(getpid)
	if [[ X"" != X"$PID" ]]; then
		kill $PID
		
		# Wait 3 seconds and check the status of the server.
		# If the server is still running kill -9 it.
		sleep 3
		PID=$(getpid)
		if [[ X"" != X"$PID" ]]; then
			echo "Server did not stop gracefully: Forcing Stop"
			kill -9 $PID
		fi
		
		echo "Done"
	else
		echo ""
		echo "Failed to stop: A running instance of the server was not found."
	fi
}

#
# Function used to start the server
#
function start {

	# Check if a JRE path is available and the correct version
	check-jre

	echo -n "Starting ${BASENAME}... "
	cd ${BASEDIR}

	# Check if there is already a running instance of the server
	PID=$(getpid)
	if [[ X"" != X"$PID" ]]; then
		echo ""
    	echo "Failed to start: An instance of the server is already running as PID ${PID}."
		echo "You must shutdown this instance before starting a new one."
		exit 1;
	fi

	# Start the server
	${JAVA_HOME}/bin/java $(tr '\n' ' ' < ${BASEDIR}/runtime.properties) $(tr '\n' ' ' < ${BASEDIR}/jvm.args) -classpath ${CLASSPATH}:${BASEDIR}/lib/*:${BASENAME}.jar com.xmlnamespace.panel.server.Server 1>>${BASEDIR}/log/${BASENAME}.out.log 2>>${BASEDIR}/log/${BASENAME}.err.log &
	
	# Write the pid file
	LAUNCHPID=$!
	echo ${LAUNCHPID} > ${BASENAME}.pid

	echo "Done"
	echo "See ${BASEDIR}/log/${BASENAME}.out.log for more details"

	# Wait for an exit value from the running server
	wait ${LAUNCHPID}
	EXITVAL=$?
	
	# Handle an upgrade
	if [ ${EXITVAL} -eq 128 ]; then
		echo "Upgrade In Progress: ${EXITVAL}" >> ${BASEDIR}/log/${BASENAME}.out.log
		rm ${BASENAME}.pid
		performUpgrade
		
	# Handle a normal exit from this script calling kill on the process	
	elif [ ${EXITVAL} -eq 143 ]; then
		echo "${BASENAME} Exiting Normally.." >> ${BASEDIR}/log/${BASENAME}.out.log
		rm ${BASENAME}.pid
		exit 0

	# Handle an abnormal exit	
	elif [ ${EXITVAL} -ne 0 ]; then
		echo "Abnormal Termination: ${EXITVAL}" >> ${BASEDIR}/log/${BASENAME}.out.log
		rm ${BASENAME}.pid
		exit 1
		
	# Handle normal exit from other sources that return exit code 0	
	else
		echo "${BASENAME} Exiting Normally.." >> ${BASEDIR}/log/${BASENAME}.out.log
		rm ${BASENAME}.pid
	fi
}

#
# Function used to query the PID of a running server instance
#
function getpid {
	local pid=`ps auxw | grep "com.xmlnamespace.panel.server.Server" | grep -v grep | awk ' { print $2 }'`
	echo "$pid"
}

#
# Function used to check maintenance expiration dates on all installed licenses against a given date
#
function license-check {

	# Check if a JRE path is available and the correct version
	check-jre

	cd ${BASEDIR}

	# Run the maintenance check and get the exit value
	${JAVA_HOME}/bin/java $(tr '\n' ' ' < ${BASEDIR}/runtime.properties) $(tr '\n' ' ' < ${BASEDIR}/jvm.args) -classpath ${CLASSPATH}:${BASEDIR}/lib/*:${BASENAME}.jar com.xmlnamespace.panel.server.Server -m "$1"
	EXITVAL=$?
	
	# Handle an exit from a maintenance expiration check
	if [ ${EXITVAL} -eq 125 ]; then
		echo "FAIL: License found with maintenance expiration before $1."
	elif [ ${EXITVAL} -eq 0 ]; then
		echo "OK: No license found with maintenance expiration before $1."

	# Handle an abnormal exit	
	elif [ ${EXITVAL} -ne 0 ]; then
		echo "Failed to check maintenance expiration: Abnormal Termination"	
	fi

	exit ${EXITVAL}
}

#
# Function used to print the version to standard out
#
function version {

	# Check if a JRE path is available and the correct version
	check-jre

	cd ${BASEDIR}

	# Run the maintenance check and get the exit value
	${JAVA_HOME}/bin/java $(tr '\n' ' ' < ${BASEDIR}/runtime.properties) $(tr '\n' ' ' < ${BASEDIR}/jvm.args) -classpath ${CLASSPATH}:${BASEDIR}/lib/*:${BASENAME}.jar com.xmlnamespace.panel.server.Server -v
	EXITVAL=$?

	exit ${EXITVAL}
}

#
# Function used to check if the jre path points to a valid jre, and if the version of the jre is supported.
#
function check-jre {

	# Check if a JRE path is available
 	if [ x${JAVA_HOME} = "x" ]; then
		echo "JAVA_HOME does not appear to be in your environment."
		echo "You must define the JAVA_HOME environment variable"
		echo "in the ${BASEDIR}/config-vars.sh shell script"
		exit 1
	else
		if [ ! -e ${JAVA_HOME}/bin/java ]; then
			echo "JAVA_HOME is defined, but the 'java' executable"
			echo "cannot be found in:  "
			echo ""
			echo "${JAVA_HOME}/bin"
			echo ""
			echo "This is a fatal error."
			echo "Verify your JAVA_HOME environment variable is set"
			echo "in ${BASEDIR}/config-vars.sh"
			exit 1
		fi
 	fi

	# Check if the JRE version is supported
	JRE_VERSION=$(${JAVA_HOME}/bin/java -version 2>&1 | awk -F '"' '/version/ {print $2}')
	if [[ "$JRE_VERSION" < "$MIN_JRE_VERSION" ]]; then
		echo "Failed to start: The server requires a Java (JRE) version of ${MIN_JRE_VERSION}, or higher"
		echo "You are currently running:"
		echo ""
		echo "${JRE_VERSION}"
		echo ""
		echo "You must upgrade Java to utilize the server"
		exit 1
	fi
}

# Command switch
case "$1" in
	"start")
		start
	;;
	"stop")
		stop
	;;
	"restart")
		stop
		start
	;;
	"getpid")
		getpid
	;;
	"version")
		version
	;;
	"license-check")
		if [[ X"" != X"${2}" ]]; then
			license-check ${2}
		else
			license-check `date +%Y-%m-%d`
		fi
	;;
	*)
		echo "Usage: ${0} {start|stop|restart|version|getpid|(license-check [date(YYYY-MM-DD)])}"
		exit 1
	;;
esac
